/****************************************************************************
 *
 * Copy right:   Qx.
 * File name:    MainTask.c
 * Description:  APP任务调取和异常处理
 * History:      2021-03-05
 * PS：主线程里面关闭了网络为了测试睡眠和唤醒
 ****************************************************************************/
#include "bsp.h"
#include "bsp_custom.h"
#include "osasys.h"
#include "ostask.h"
#include "queue.h"
#include "ps_event_callback.h"
#include "cmisim.h"
#include "cmimm.h"
#include "cmips.h"
#include "sockets.h"
#include "psifevent.h"
#include "ps_lib_api.h"
#include "lwip/netdb.h"
#include <cis_def.h>
#include "debug_log.h"
#include "slpman_ec616.h"
#include "plat_config.h"
#include "ec_tcpip_api.h"
#include "hal_module_adapter.h"
#include "timers.h"
#include "app.h"
#include "MainTask.h"



//全局变量
UINT32 Timer_count;//每100ms加1
volatile bool Sleep_flag = false;//睡眠标志位


//主线程堆栈声明区
static StaticTask_t           gProcess_Main_Task_t;
static UINT8                  gProcess_Main_TaskStack[PROC_MAIN_TASK_STACK_SIZE];
static osThreadId_t           MainTaskId = NULL;
uint8_t MainSlpHandler          = 0xff;//主线程睡眠句柄
uint8_t deepslpTimerID          = DEEPSLP_TIMER_ID7;//睡眠定时器ID
volatile bool Work_timer_end = false;
static process_Main 		    gProcess_Main_Task;
#define PROC_MAIN_STATE_SWITCH(a)  (gProcess_Main_Task = a)

//函数声明区
static void appBeforeHib(void *pdata, slpManLpState state);
static void appAfterHib(void *pdata, slpManLpState state);
static void appBeforeSlp1(void *pdata, slpManLpState state);
static void appAfterSlp1(void *pdata, slpManLpState state);
static void appBeforeSlp2(void *pdata, slpManLpState state);
static void appAfterSlp2(void *pdata, slpManLpState state);
void montior_timer_callback(TimerHandle_t xTimer);
void work_timer_callback(TimerHandle_t xTimer);



//主线程任务区
static void MainTask(void* arg)
{

    appSetEDRXSettingSync(0,5,1800000);
    appSetPSMSettingSync(0,3*60*60,10);
    appSetCFUN(0);

    int32_t inParam = 0xAABBCCDD;
    slpManSetPmuSleepMode(true,SLP_HIB_STATE,false);
    slpManApplyPlatVoteHandle("MainSlp",&MainSlpHandler);
    slpManPlatVoteDisableSleep(MainSlpHandler, SLP_SLP1_STATE);
    slpManRegisterUsrdefinedBackupCb(appBeforeHib,&inParam,SLPMAN_HIBERNATE_STATE);
    slpManRegisterUsrdefinedRestoreCb(appAfterHib,NULL,SLPMAN_HIBERNATE_STATE);
    slpManRegisterUsrdefinedBackupCb(appBeforeSlp1,NULL,SLPMAN_SLEEP1_STATE);
    slpManRegisterUsrdefinedRestoreCb(appAfterSlp1,NULL,SLPMAN_SLEEP1_STATE);
    slpManRegisterUsrdefinedBackupCb(appBeforeSlp2,NULL,SLPMAN_SLEEP2_STATE);
    slpManRegisterUsrdefinedRestoreCb(appAfterSlp2,NULL,SLPMAN_SLEEP2_STATE);
    slpManSlpState_t slpstate = slpManGetLastSlpState(); 

    TimerHandle_t montior_timer = NULL;
    TimerHandle_t work_timer = NULL;
    slpManWakeSrc_e Wakeup_source;
    
    static UINT8 work_time;
    static UINT8 Sleep_Time;
    Sleep_Time = 1;//睡眠时长 min
    work_time = 1;//工作时长 min

    montior_timer = xTimerCreate("montior_timer", 100 / portTICK_RATE_MS, pdTRUE, NULL, montior_timer_callback);
    work_timer = xTimerCreate("work_timer", work_time*60*1000 / portTICK_RATE_MS, pdFALSE, NULL, work_timer_callback);
    PROC_MAIN_STATE_SWITCH(PROCESS_STATE_IDLE);
    while(true)
    {
        switch(gProcess_Main_Task)
        {
            case PROCESS_STATE_IDLE:
            {
                Timer_count = 0;
                xTimerStart(montior_timer, 0);
                xTimerStart(work_timer, 0);
                Sleep_flag = false;
                PROC_MAIN_STATE_SWITCH(PROCESS_STATE_WORK);
                break;
            }
            case PROCESS_STATE_WORK:
            {
                osDelay(1000);
                if(Work_timer_end)
                {
                    PROC_MAIN_STATE_SWITCH(PROCESS_STATE_SLEEP);
                }
                break;
            }
            case PROCESS_STATE_SLEEP:
            {
                Sleep_flag = true;
                xTimerStop(montior_timer, 0);
                xTimerStop(work_timer, 0);
                slpManSlpState_t State;
                uint8_t cnt;
                if(slpManCheckVoteState(MainSlpHandler, &State, &cnt)==RET_TRUE)
                {
                    #ifdef USING_PRINTF
                        printf("[%d]We Can Check Vote Main State, state=%d, cnt=%d\r\n",__LINE__,State,cnt);
                    #endif						
                }
                slpManPlatVoteForceEnableSleep(MainSlpHandler, SLP_SLP1_STATE);
                if(slpManCheckVoteState(MainSlpHandler, &State, &cnt)==RET_TRUE)
                {
                    #ifdef USING_PRINTF
                        printf("[%d]We Can Check Vote Main State Again, state=%d, cnt=%d\r\n",__LINE__,State,cnt);
                    #endif					
                }
                #ifdef USING_PRINTF
                    printf("[%d]Waiting sleep\r\n",__LINE__);
                #endif
                FaultDisplay(LED_TURN_ON);
                slpManSlpState_t slpstate = slpManPlatGetSlpState();
                #ifdef USING_PRINTF
                    printf("which slpstate can go now :%d \n",slpstate);
                #endif
                slpManDeepSlpTimerStart(deepslpTimerID, Sleep_Time*1000*60);
                while(1)
                {
                    osDelay(20*60*1000/portTICK_PERIOD_MS);
                    break;
                }
                PROC_MAIN_STATE_SWITCH(PROCESS_STATE_IDLE);
                break;
            }
            default:
            {
                PROC_MAIN_STATE_SWITCH(PROCESS_STATE_IDLE);
                #ifdef USING_PRINTF
                    printf("default!\n");
                #endif
                break;
            }
        }
    }
}
void MainTaskInit(void *arg)
{
    #ifndef USING_PRINTF
    if(BSP_GetPlatConfigItemValue(PLAT_CONFIG_ITEM_LOG_CONTROL) != 0)
    {
        HAL_UART_RecvFlowControl(false);
    }
    #endif
    osThreadAttr_t task_attr;
    memset(&task_attr,0,sizeof(task_attr));
    memset(gProcess_Main_TaskStack, 0xA5, PROC_MAIN_TASK_STACK_SIZE);
    task_attr.name = "Main_Task";
    task_attr.stack_mem = gProcess_Main_TaskStack;
    task_attr.stack_size = PROC_MAIN_TASK_STACK_SIZE;
    task_attr.priority = osPriorityNormal;
    task_attr.cb_mem = &gProcess_Main_Task_t;
    task_attr.cb_size = sizeof(StaticTask_t);
    MainTaskId = osThreadNew(MainTask, NULL, &task_attr);

}
void MainTaskDeInit(void *arg)
{
    osThreadTerminate(MainTaskId);
    MainTaskId = NULL;
}
//定时器回调函数区
void montior_timer_callback(TimerHandle_t xTimer)
{
    #ifdef USING_PRINTF
        if (Timer_count%50==0)
        {
            printf("Main Task,Batt_Cell_Num:%d,%d!\n",BATT_CELL_VOL_NUM,Timer_count);
        }
    #endif
    Timer_count++;
    if(Timer_count>100000*100)
    {
        Timer_count=0;
    }
}
void work_timer_callback(TimerHandle_t xTimer)
{
    Work_timer_end = true;
    #ifdef USING_PRINTF
        printf("Hello work Timer!\n");
    #endif
}

//睡眠进出函数区
static void appBeforeHib(void *pdata, slpManLpState state)
{
    uint32_t *p_param = (uint32_t *)pdata;

    #ifdef USING_PRINTF
        printf("Before Hibernate:%d \n",state);
    #endif
	slpManAONIOLatchEn(AonIOLatch_Enable);

}
static void appAfterHib(void *pdata, slpManLpState state)
{
    #ifdef USING_PRINTF
        printf("Try Hibernate Failed:%d \n",state);
    #endif
}
static void appBeforeSlp1(void *pdata, slpManLpState state)
{
    #ifdef USING_PRINTF
        printf("Before Sleep1:%d \n",state);
    #endif
	slpManAONIOLatchEn(AonIOLatch_Enable);
}
static void appAfterSlp1(void *pdata, slpManLpState state)
{
    #ifdef USING_PRINTF
        printf("After Sleep1:%d \n",state);
    #endif
}
static void appBeforeSlp2(void *pdata, slpManLpState state)
{
    #ifdef USING_PRINTF
        printf("before sleep2:%d \n",state);
    #endif
	slpManAONIOLatchEn(AonIOLatch_Enable);
}
static void appAfterSlp2(void *pdata, slpManLpState state)
{
    #ifdef USING_PRINTF
        printf("sleep2 failed:%d \n",state);
    #endif
}



